'==============================================================================
' Maximite program to Clock Control 3 x Stepper Motor Controllers.
' This board will control Speed, Steps and Fwd/Rev I/O.
' Capable of Pulse speeds upto 5kHz
' Rx Cmnd   = "Mn,Speed,Steps,Cmnd"
'                     where Mn      = 'Mn' = Motor number
'                           Speed   = Clocking Frequency
'                           Steps   = Number of Steps to clock at this Speed.
'                           Cmnd    = StepNbr to execute
' TxStr     = "n,Speed,Steps,RxStep,OK
'             "0,00000,00000,00    ,OK"
'                     where RxStep  = Step it was told to goto.
' Possible Commands:
'                   Reset       = 1  Reset Counters and Steps
'                   Fwd.        = 2  Set Speed & Target Count to go FWD.
'                   Rev.        = 3  Set Speed & Target Count to go REV.
'                   Stop        = 8  Stop the Clock, but leave the count as is.
'                                    goes back to Step -1.Resend Freq only to restart.
'                   Input       = 9  Get Value for that Counter, goes back to last step.
' 
' To Test send "M1,00000,00000,01"  Reset               Reply="1,00000,00000,-1,OK"
'              "M1,00100,00200,02"  Target Fwd Steps    Reply="1,00100,00200, 2,OK"
'              "M1,00100,00200,03"  Target Rev Steps    Reply="1,00100,00200, 3,OK"
'              "M1,00000,00000,09"  Get count update    Reply="1,00100,?????, 9,OK"
' when count complete this version will reply with            "1,?????,?????, 5,Fi" (Finished)  
'NOTES:     If all Counters are Idle, and Timer > 90000 then Timer is set to 0.
'           this prevent overflow if run for extended periods.
'   Pin(1) = Clk Count Input for Stepper Motor 1 (Use 330 ohm resistor to Pin 7)
'   Pin(2) = Clk Count Input for Stepper Motor 2 (Use 330 ohm resistor to Pin 8)
'   Pin(3) = Clk Count Input for Stepper Motor 3 (Use 330 ohm resistor to Pin 9)
'   Pin(7) = PWM Output to Clk Motor 1 & input to Pin(1)
'   Pin(8) = PWM Output to Clk Motor 1 & input to Pin(2)
'   Pin(9) = PWM Output to Clk Motor 1 & input to Pin(3)
'   Pin(11)= Rev Output to Motor 1
'   Pin(12)= Rev Output to Motor 2
'   Pin(13)= Rev Output to Motor 3
'==============================================================================
new
10  Option Base 1
'   Set pins 1 - 3 as unconfigured (will be set to interrupt on count on low to high transition at step 3)
20  For i=1 To 3
21   SetPin i,0
22  Next i
'   Pins 7-9 will be the PWM outputs.
'   Pins 11-20 will be outputs
30  For i=11 To 20
31   Pin(i)=0:SetPin i,8
32  Next i
'   Program variables
40  S1=0:S2=0:S3=0 ' Sn: Current step number
41  F1=0:F2=0:F3=0 ' Tn: Target Freq
42  N1=0:N2=0:N3=0 ' Tn: Target Steps
43  O1=0:O2=0:O3=0 ' On: Old count
44  Z1=0:Z2=0:Z3=0 ' Zn: Old Step to allow going back.
45  M=0      ' Input state machine value
46  Timer=0
'   Start main loop++++++++++++++++++++++++++++++++++++++++++++++++
100 Do
'    Command input routine start
110  K$=UCase$(InKey$)
120  If K$<>"" Then
130    If K$="Q" Then End
140    If K$="M" Then
150      Line Input RxStr$
160      If Len(RxStr$) >= 16 Then
'           M=Motor,F=Freq,N=Nbr Steps,S=Step "M0,00000,00000,00"
170        M=Val(Left$(RxStr$,1))
180        F=Val(Mid$(RxStr$,3,5))
190        N=Val(Mid$(RxStr$,9,5))
200        S=Val(Mid$(RxStr$,15,2))
210        If M>0 And M<4 Then
220          If M=1 Then
230              F1=F:N1=N:S1=S
240            ElseIf M=2 Then
250              F2=F:N2=N:S2=S
260            ElseIf M=3 Then
270              F3=F:N3=N:S3=S
280          EndIf
290        EndIf
300      EndIf
310    EndIf
320  EndIf
'    Command input routine end
'    Pulser processing routine
'     Process channel 1
400  If Idl=3 Then
410    Idl=0
420    If Timer > 90000 Then 
430      Timer=0
440    EndIf
450  EndIf

1000  Do
1010   If S1=-1 Then
1020    Idl=1:Z1=-1:Exit
1030   ElseIf S1=0 Then
1040    ? Format$(S1,"1,00000,00000,%02.0f,OK"):S1=-1:Exit
1050   ElseIf S1=1 Then
1060    Z1=1:F1=0:N1=0
1070    ? Format$(Z1,"1,00000,00001,%02.0f,OK"):S1=-1:Exit
'       STEP 2 GO FWD.Pin11 = 0
1080   ElseIf S1=2 Then
1090    If F1=0 Then
1100      Z1=2:? Format$(F1,"1,%05.0f,");Format$(N1,"%05.0f,");Format$(Z1,"%02.0f,Er"):S1=-1:Exit
1110    EndIf
1120    SetPin 7,10,F1,50:Pin(11)=0:If N1>0 Then SetPin 1,11,10000,N1,2
1130    Z1=2:? Format$(F1,"1,%05.0f,");Format$(N1,"%05.0f,");Format$(Z1,"%02.0f,OK"):S1=4:Exit
'       STEP 3 GO REV. Pin 11 = 1
1140   ElseIf S1=3 Then
1150    If F1=0 Then
1160      Z1=3:? Format$(F1,"1,%05.0f,");Format$(N1,"%05.0f,");Format$(Z1,"%02.0f,Er"):S1=-1:Exit
1170    EndIf
1180    SetPin 7,10,F1,50:Pin(11)=1:If N1>0 Then SetPin 1,11,10000,N1,2
1190    Z1=3:? Format$(F1,"1,%05.0f,");Format$(N1,"%05.0f,");Format$(Z1,"%02.0f,OK"):S1=4:Exit
'       WAIT for Terminal Count
1200   ElseIf S1=4 Then
1210    Z1=4:Exit
1220   ElseIf S1=5 Then 
1230    Z1=5:? Format$(F1,"1,%05.0f,");Format$(O1,"%05.0f,");Format$(Z1,"%02.0f,Fi")
1240    SetPin 1,0:S1=-1:F1=0:N1=0:Exit

1250   ElseIf S1=8 Then
1260    Z1=8:SetPin 7,8:F1=0
1270    ? Format$(F1,"1,%05.0f,");Format$(PinCnt(1),"%05.0f,");Format$(Z1,"%02.0f,OK"):S1=-1:Exit
1280   ElseIf S1=9 Then
1290    ? Format$(F1,"1,%05.0f,");Format$(PinCnt(1),"%05.0f,");Format$(Z1,"%02.0f,OK")
1300    S1=Z1:Exit
1310   Else
1320    S1=-1:Exit    
1330   EndIf
1340   Exit
1350  Loop

2000  Do
2010   If S2=-1 Then
2020    Idl=Idl+1:Z2=-1:Exit
2030   ElseIf S2=0 Then
2040    ? Format$(S2,"2,00000,00000,%02.0f,OK"):S2=-1:Exit
2050   ElseIf S2=1 Then
2060    Z2=1:F2=0:N2=0
2070    ? Format$(Z2,"2,00000,00000,%02.0f,OK"):S2=-1:Exit
'       STEP 2 GO FWD.Pin12 = 0
2080   ElseIf S2=2 Then
2090    If F2=0 Then
2100      Z2=2:? Format$(F2,"2,%05.0f,");Format$(N2,"%05.0f,");Format$(Z2,"%02.0f,Er"):S2=-1:Exit
2110    EndIf
2120    SetPin 8,10,F2,50:Pin(12)=0:If N2>0 Then SetPin 2,11,20000,N2,2
2130    Z2=2:? Format$(F2,"2,%05.0f,");Format$(N2,"%05.0f,");Format$(Z2,"%02.0f,OK"):S2=4:Exit
'       STEP 3 GO REV. Pin 12 = 1
2140   ElseIf S2=3 Then
2150    If F2=0 Then
2160      Z2=3:? Format$(F2,"2,%05.0f,");Format$(N2,"%05.0f,");Format$(Z2,"%02.0f,Er"):S2=-1:Exit
2170    EndIf
2180    SetPin 8,10,F2,50:Pin(12)=1:If N2>0 Then SetPin 2,11,20000,N2,2
2190    Z2=3:? Format$(F2,"2,%05.0f,");Format$(N2,"%05.0f,");Format$(Z2,"%02.0f,OK"):S2=4:Exit
'       WAIT for Terminal Count
2200   ElseIf S2=4 Then
2210    Z2=4:Exit
2220   ElseIf S2=5 Then 
2230    Z2=5:? Format$(F2,"2,%05.0f,");Format$(O2,"%05.0f,");Format$(Z2,"%02.0f,Fi")
2240    SetPin 2,0:S2=-1:F2=0:N2=0:Exit

2250   ElseIf S2=8 Then
2260    Z2=8:SetPin 8,8:F2=0
2270    ? Format$(F2,"2,%05.0f,");Format$(PinCnt(1),"%05.0f,");Format$(Z2,"%02.0f,OK"):S2=-1:Exit
2280   ElseIf S2=9 Then
2290    ? Format$(F2,"2,%05.0f,");Format$(PinCnt(1),"%05.0f,");Format$(Z2,"%02.0f,OK")
2300    S2=Z2:Exit
2310   Else
2320    S2=-1:Exit    
2330   EndIf
2340   Exit
2350  Loop

3000  Do
3010   If S3=-1 Then
3020    Idl=Idl+1:Z3=-1:Exit
3030   ElseIf S3=0 Then
3040    ? Format$(S3,"3,00000,00000,%02.0f,OK"):S3=-1:Exit
3050   ElseIf S3=1 Then
3060    Z3=1:F3=0:N3=0
3070    ? Format$(Z3,"3,00000,00000,%02.0f,OK"):S3=-1:Exit
'       STEP 2 GO FWD.Pin13 = 0
3080   ElseIf S3=2 Then
3090    If F3=0 Then
3100      Z3=2:? Format$(F3,"3,%05.0f,");Format$(N3,"%05.0f,");Format$(Z3,"%02.0f,Er"):S3=-1:Exit
3110    EndIf
3120    SetPin 9,10,F3,50:Pin(13)=0:If N3>0 Then SetPin 3,11,30000,N3,2
3130    Z3=2:? Format$(F3,"3,%05.0f,");Format$(N3,"%05.0f,");Format$(Z3,"%02.0f,OK"):S3=4:Exit
'       STEP 3 GO REV. Pin 13 = 1
3140   ElseIf S3=3 Then
3150    If F3=0 Then
3160      Z3=3:? Format$(F3,"3,%05.0f,");Format$(N3,"%05.0f,");Format$(Z3,"%02.0f,Er"):S3=-1:Exit
3170    EndIf
3180    SetPin 9,10,F3,50:Pin(13)=1:If N3>0 Then SetPin 3,11,30000,N3,2
3190    Z3=3:? Format$(F3,"3,%05.0f,");Format$(N3,"%05.0f,");Format$(Z3,"%02.0f,OK"):S3=4:Exit
'       WAIT for Terminal Count
3200   ElseIf S3=4 Then
3210    Z3=4:Exit
3220   ElseIf S3=5 Then 
3230    Z3=5:? Format$(F3,"3,%05.0f,");Format$(O3,"%05.0f,");Format$(Z3,"%02.0f,Fi")
3240    SetPin 3,0:S3=-1:F3=0:N3=0:Exit

3250   ElseIf S3=8 Then
3260    Z3=8:SetPin 9,8:F3=0
3270    ? Format$(F3,"3,%05.0f,");Format$(PinCnt(1),"%05.0f,");Format$(Z3,"%02.0f,OK"):S3=-1:Exit
3280   ElseIf S3=9 Then
3290    ? Format$(F3,"3,%05.0f,");Format$(PinCnt(1),"%05.0f,");Format$(Z3,"%02.0f,OK")
3300    S3=Z3:Exit
3310   Else
3320    S3=-1:Exit    
3330   EndIf
3340   Exit
3350  Loop
9999 Loop
'     Counter Int Routines
10000 If S1=4 Then
10010  SetPin 7,8:O1=Pincnt(1):S1=5
10020 EndIf:IReturn
20000 If S2=4 Then
20010  SetPin 8,8:O2=Pincnt(2):S2=5
20020 EndIf:IReturn
30000 If S3=4 Then
30010  SetPin 9,8:O3=Pincnt(3):S3=5
30020 EndIf:IReturn
